;******************************************************************************
;* Name:        dmx_send.asm
;* Funktion:    Sender fuer das DMX-Protokoll fuer
;*                512 Kanaele DMX-IN
;*                512 Kanaele DMX-OUT
;*
;*          [PC-512 Kanaele]-->
;*                              [HTP]--->[512 Kan. DMX-OUT]
;*          [DMX-IN 512Kan.]-->
;*
;*-----------------[ RS485 - Transmitter ]--------------------------------
;*
;*                           |+\---------O DMXP(3)
;*              DMX_OUT >----|U4>
;*              P3.1/TxD     |-/o--------O DMXN(4)
;*
;*         DMXP(1)o---------o|+\
;*                           |U4>--------o DMX_IN
;*         DMXN(2)o----------|-/           P3.1/RxD
;*
;* SUB-D9F:  1=DMXP-IN  2=DMXN-IN  3=DMXP-OUT  4=DMXN-OUT  5=+9..+15V
;*           6..9=GND
;*
;*-----[ Serielle Schnittstelle einrichten ]---------
;* Geschwindigkeit:      250KBit/s
;* Teilungsfaktor:       / 64
;* Vorteiler(SMOD):      ON
;* Timer0-Wert:          1
;* Mode:                 2
;* /2 -> /2 -> /16
;*
;* Pin | Stecker-Bez | Geraet-Bez. |Reg# Bit#
;*-----+-------------+-------------+----+----
;*  1  | /Strobe     | P3.2/INT0   | 2  | 0
;*  2  | D0          | P1.0        | 0  | 0
;*  3  | D1          | P1.1        | 0  | 1
;*  4  | D2          | P1.2        | 0  | 2
;*  5  | D3          | P1.3        | 0  | 3
;*  6  | D4          | P1.4        | 0  | 4
;*  7  | D5          | P1.5        | 0  | 5
;*  8  | D6          | P1.6        | 0  | 6
;*  9  | D7          | P1.7        | 0  | 7
;*  10 | ACKN        | P3.7        | 1  | 0
;*  11 | Busy        | P3.5        | 1  | 1
;*  14 | /Autofeed   | P3.3/INT1   | 2  | 1
;*  16 | INIT        | ---         | 2  | 2
;*  17 | /SELECT     | ---         | 2  | 3
;*  18 | GND         | GND         | -
;*  .. | ....        | ....        |
;*  25 | GND         | GND         | -
;******************************************************************************

;==========[ Einstellungen ]===================================
.equ VERSION, 31
.equ DMXBREAK, 16*88

; die Anzahl zu uebertragender Kanaele (Bei SIZE_IN_RAM=1 nur default)
.equ KANALZAHL, 110    ; Anzahl der zu uebertragenden Kanaele
.equ DMXKANAL1, 004h   ; 110 Kanaele ab hier  (ab R4)
.equ DMXKANALN, (DMXKANAL1+KANALZAHL-1)
.equ STACKSTART, (DMXKANAL1+KANALZAHL)

.flag DMX_OUT,  P3.1
.flag DATA_LED, P3.4
.flag BUSY,     P3.5
.flag ACN,      P3.7

.equ LPT_DATA, P1

.org 0h
        ljmp    Start

; exec Time = ca. 5us,4us,2us + Interrupt-overhead
.ORG 3H          ; EXT0 - P3.2 - Data-Transfer-Request
        setb    ACN
        cjne    R0,#STACKSTART,L01
L02:    clr     ACN
        reti
L01:    
        ;push    A ; das geht nicht mit as31
        mov     A,LPT_DATA
        movx    @R0,A
        ;pop     A ; geht nicht mit as31

        inc     R0
        clr     DATA_LED
        sjmp    L02

; exec Time = ca. 3.5us + Interupt-overhead
.ORG 13H         ; EXT1-P3.3
        setb    ACN
        mov     R0,#DMXKANAL1
        clr     ACN
        reti

.ORG 23H         ; Serieller Port
        reti

.ORG 2BH         ; Timer2 oder EXT2
        reti

;       mov     IE,#10000000b
define(`KKA_Begin', `
        clr     EA
')

;       mov     IE,#10000101b
define(`KKA_End', `
        setb    EA
')

define(`WAITus', `
        KKA_End()
        mov     TH0,#0FFh - (((16*$1)/12) / 100h)
        mov     TL0,#0FFh - (((16*$1)/12) & 0FFh)
        clr     TF0
        setb    TR0
        jnb     TF0,*
        clr     TR0
        KKA_Begin()
')

define(`WAITms', `
        KKA_End()
        mov     R2,#$1
waitms1:mov     TH0,#0FFh - ((16000/12) / 100h)
        mov     TL0,#0FFh - ((16000/12) & 0FFh)
        clr     TF0
        setb    TR0
        jnb     TF0,*
        clr     TR0
        djnz    R2,waitms1
        KKA_Begin()
')


;==========[ DMX-Interface initiallisieren ]===================================
Start:  mov     PCON,  #00000000b       ; Power-Register - Wakeup
        mov     PSW,   #00000000b       ; REGISTERBANK_0 initialisieren
        mov     SP,    #STACKSTART      ; Stackanfang hinter den letzten Kanal
                                        ; 110 Kanaele
        mov     IP,    #00010000b       ; setze Prioritt
        mov     IE,    #00000000b

;        clr     PCON.7
        MOV     A,     PCON             ; Make sure SMOD is 0N.
        clr     ACC.7
        MOV     PCON,  A

        MOV     SCON,  #10000000b       ; Set up serial port.
        mov     TMOD,  #00100001b   ; Timer-Modus setzen
        mov     TCON,  #00000101b   ; IRQ Flanke/Pegel   / Flanke an INT0,INT1

        mov     LPT_DATA,#0FFh
        setb    P3.2 ;INTR0
        setb    P3.3 ;INTR1
        clr     ACN
        clr     BUSY

        mov     R0,#DMXKANAL1
        setb    EX0
        setb    EX1
        setb    EA
;       mov     IE,#10000101b
        setb    TI

; Alle Kaneale auf 0 setzen
        mov     R1,#DMXKANAL1
        mov     R3,#KANALZAHL
initl0:
;        movx    @R1,#00h
        mov     A,R1
        movx    @R1,A

        inc     R1
        djnz    R3,initl0

;==================[ Hauptschleife ]====================
loop:
        KKA_Begin()
        clr     DMX_OUT
        WAITus(90)
        setb    DMX_OUT
        nop    ; ca. 4us Mark after Break
        nop   
        nop
        nop
        nop
        nop
        nop
        nop
        setb    TB8
        mov     SBUF,#0
        KKA_End

        mov     R1,#DMXKANAL1
        mov     R3,#KANALZAHL
serl:   jnb     TI,serl
        clr     TI

        setb    TB8
        movx    A,@R1
        mov     SBUF,A
        inc     R1
        djnz    R3,serl

        jnb     TI,*  ; warten bis das letzte Byte uebertragen wurde
        clr     TI

        KKA_Begin()
        WAITms(4)
        KKA_End()

        setb    DATA_LED

        ljmp   loop
